"""
Turn left.
Turn right.
Turn toward the pyramid.
Turn toward the stalactite.
Move forward.
Move back. (5)
Move left. 
Move right.
Move before the red stalactite.
Move before the blue stalactite.
Move before the red pyramid.  (10)
Move before the blue pyramid.
Go to the pyramid.
Go to the stalactite.
Go to the door.
Go to the red pyramid.  (15)
Go to the red stalactite.
Go to the blue pyramid.
Go to the blue stalactite.
Centre the pyramid.
Centre the stalactite. (20)
Explore. (21)
Stop. (22)

WORDS 
Back, Blue, Before, Centre, Door, Explore,  (5)
Forward, Go, Left, Move, Period, (10)
Pyramid, Red, Right, Stalactite, Stop, (15)
The, To, Toward, Turn (19)

Final States
3 (turn left), 5 (turn right), 9 (turn toward the pyramid)
11 (turn toward the stalactite), 14 (move forward), 
16 (move back), 18 (move left), 20 (move right), 
25 (move before the red stalactite), 28 (move before the blue stalactite)
30 (move before the red pyramid), 32 (move before the blue pyramid)
37 (go to the pyramid), 39 (go to the stalactite), 41 (go to the door)
44 (go to the red pyramid), 46 (go to the red stalactite)
49 (go to the blue pyramid), 51 (go to the blue stalactite)
55 (centre the pyramid), 57 (centre the stalactite), 
59 (explore), 61 (stop)

STATES
Start, Turn, TurnLeft,TurnLeftP,  TurnRight, TurnRightP, (5)
TurnToward, TurnTowardThe, TurnTowardThePyramid, TurnTowardThePyramidP, 
TurnTowardTheStalactite,  (10)
TurnTowardTheStalactiteP,  Move, MoveForward, MoveForwardP,    MoveBack (15)
MoveBackP, MoveLeft, MoveLeftP,     MoveRight, MoveRightP, (20)
MoveBefore, MoveBeforeThe,  MoveBeforeTheRed
MoveBeforeTheRedStalactite, MoveBeforeTheRedStalactiteP, (25)
MoveBeforeTheBlue, MoveBeforeTheBlueStalactite, 
MoveBeforeTheBlueStalactiteP, MoveBeforeTheRedPyramid, 
MoveBeforeTheRedPyramidP, (30)
MoveBeforeTheBluePyramid, MoveBeforeTheBluePyramidP, Go, GoTo, GoToThe (35)
GoToThePyramid, GoToThePyramidP,GoToTheStalactite, GoToTheStalactiteP
GoToTheDoor, (40)
GoToTheDoorP, GoToTheRed, GoToTheRedPyramid, GoToTheRedPyramidP,
GoToTheRedStalactite, (45)
GoToTheRedStalactiteP, GoToTheBlue, GoToTheBluePyramid, GoToTheBluePyramidP,
GoToTheBlueStalactite, (50)
GoToTheBarredStalact, oCentre, CentreThe,CentreThePyramid, CentreThePyramidP (55)
CentreTheStalactite, CentreTheStalactiteP, Explore,  ExploreP, Stop, (60)
StopP (61)


"""


#---- Code
import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
    import spynnaker_external_devices_plugin.pyNN as externaldevices
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

import os.path

class ParseClass:
    NUMBER_WORDS = 20
    NUMBER_STATES = 62
    def __init__(self, simName):
        self.simulator = simName
        self.neal = NealCoverFunctions(self.simulator)
        self.fsa = FSAHelperFunctions(self.simulator)
        self.fsa.initParams()
        parseCells = self.createNeurons()
        self.stateCells = parseCells[0]
        self.wordCells = parseCells[1]
        self.setupRecording()
        self.makeAutomaton()

    #create the neurons for the words and the states.
    def createNeurons(self):
        numberWordCells = self.NUMBER_WORDS * self.fsa.CA_SIZE
        numberStateCells = self.NUMBER_STATES * self.fsa.CA_SIZE
        localStateCells=sim.Population(numberStateCells,sim.IF_cond_exp,
                    self.fsa.CELL_PARAMS)
        localWordCells=sim.Population(numberWordCells,sim.IF_cond_exp,
                    self.fsa.CELL_PARAMS)
        
        return [localStateCells,localWordCells]

    def makeWordCAs(self):
        for wordNumber  in range (0,self.NUMBER_WORDS):
            self.fsa.makeCA(self.wordCells,wordNumber)

    def makeStates(self):
        for stateNumber  in range (0,self.NUMBER_STATES):
            self.fsa.makeCA(self.stateCells,stateNumber)

    def stateHalfStartsState(self, fromState,toState):
        self.fsa.stateHalfTurnsOnState(self.stateCells,fromState,
                                       self.stateCells,toState)
        self.fsa.stateTurnsOffState(self.stateCells,toState,self.stateCells,
                                    fromState)

    def wordHalfStartsState(self, fromState,toState):
        self.fsa.stateHalfTurnsOnState(self.wordCells,fromState,
                                       self.stateCells,toState)
        self.fsa.stateTurnsOffState(self.stateCells,toState,self.wordCells,
                                    fromState)

    def stateAndWordSwitchState(self,initState,finalState,word):
        self.stateHalfStartsState(initState,finalState)
        self.wordHalfStartsState(word,finalState)
        
    def makeAutomaton(self):
        self.makeWordCAs()
        self.makeStates()

        #turn left
        self.stateAndWordSwitchState(0,1,19)
        self.stateAndWordSwitchState(1,2,8)
        self.stateAndWordSwitchState(2,3,10)

        #TURN right .
        self.stateAndWordSwitchState(1,4,13)
        self.stateAndWordSwitchState(4,5,10)

        #TURN toward the pyramid .
        self.stateAndWordSwitchState(1,6,18)
        self.stateAndWordSwitchState(6,7,16)
        self.stateAndWordSwitchState(7,8,11)
        self.stateAndWordSwitchState(8,9,10)

        #TURN TOWARD THE STALACTITE .
        self.stateAndWordSwitchState(7,10,14)
        self.stateAndWordSwitchState(10,11,10)

        #move forward .
        self.stateAndWordSwitchState(0,12,9)
        self.stateAndWordSwitchState(12,13,6)
        self.stateAndWordSwitchState(13,14,10)

        #MOVE back .
        self.stateAndWordSwitchState(12,15,0)
        self.stateAndWordSwitchState(15,16,10)

        #MOVE left .
        self.stateAndWordSwitchState(12,17,8)
        self.stateAndWordSwitchState(17,18,10)

        #MOVE right .
        self.stateAndWordSwitchState(12,19,13)
        self.stateAndWordSwitchState(19,20,10)

        #MOVE before the red stalactite .
        self.stateAndWordSwitchState(12,21,2)
        self.stateAndWordSwitchState(21,22,16)
        self.stateAndWordSwitchState(22,23,12)
        self.stateAndWordSwitchState(23,24,14)
        self.stateAndWordSwitchState(24,25,10)

        #MOVE BEFORE THE blue stalactite .
        self.stateAndWordSwitchState(22,26,1)
        self.stateAndWordSwitchState(26,27,14)
        self.stateAndWordSwitchState(27,28,10)

        #MOVE BEFORE THE RED pyramid .
        self.stateAndWordSwitchState(23,29,11)
        self.stateAndWordSwitchState(29,30,10)

        #MOVE BEFORE THE BLUE pyramid .
        self.stateAndWordSwitchState(26,31,11)
        self.stateAndWordSwitchState(31,32,10)

        #""" kills docker nest undone
        #go to the pyramid .
        self.stateAndWordSwitchState(0,33,7)
        self.stateAndWordSwitchState(33,34,17)
        self.stateAndWordSwitchState(34,35,16)
        self.stateAndWordSwitchState(35,36,11)
        self.stateAndWordSwitchState(36,37,10)
        

        #GO TO THE stalactite .
        self.stateAndWordSwitchState(35,38,14)
        self.stateAndWordSwitchState(38,39,10)

        #GO TO THE door .
        self.stateAndWordSwitchState(35,40,4)
        self.stateAndWordSwitchState(40,41,10)

        #GO TO THE red pyramid .
        self.stateAndWordSwitchState(35,42,12)
        self.stateAndWordSwitchState(42,43,11)
        self.stateAndWordSwitchState(43,44,10)

        #GO TO THE RED stalactite .
        self.stateAndWordSwitchState(42,45,14)
        self.stateAndWordSwitchState(45,46,10)

        #GO TO THE blue pyramid .
        self.stateAndWordSwitchState(35,47,1)
        self.stateAndWordSwitchState(47,48,11)
        self.stateAndWordSwitchState(48,49,10)

        #GO TO THE BLUE stalactite .
        self.stateAndWordSwitchState(47,50,14)
        self.stateAndWordSwitchState(50,51,10)

        #centre the pyramid .
        self.stateAndWordSwitchState(0,52,3) #centre
        self.stateAndWordSwitchState(52,53,16) #the
        self.stateAndWordSwitchState(53,54,11) #pyramid
        self.stateAndWordSwitchState(54,55,10)

        #CENTRE THE stalactite .
        self.stateAndWordSwitchState(53,56,14) #stalactite
        self.stateAndWordSwitchState(56,57,10)

        #explore .
        self.stateAndWordSwitchState(0,58,5) #explore
        self.stateAndWordSwitchState(58,59,10)

        #stop .
        self.stateAndWordSwitchState(0,60,15) #stop
        self.stateAndWordSwitchState(60,61,10)
        #"""

    def setupRecording(self):
        if  (nealParameters.simulator == 'nest'):
            self.stateCells.record({'spikes','v'})
            self.wordCells.record({'spikes','v'})
        elif  (nealParameters.simulator == 'spinnaker'):
            self.stateCells.record()
            self.wordCells.record()

    def printPklSpikes(self,fileName):
        fileHandle = open(fileName)
        neoObj = pickle.load(fileHandle)
        segments = neoObj.segments
        segment = segments[0]
        spikeTrains = segment.spiketrains
        neurons = len(spikeTrains)
        for neuronNum in range (0,neurons):
            if (len(spikeTrains[neuronNum])>0):
                spikes = spikeTrains[neuronNum]
                for spike in range (0,len(spikes)):
                    print neuronNum, spikes[spike]
        fileHandle.close()
    
    def printResults(self):
        if (nealParameters.simulator == 'nest'):
            self.stateCells.printSpikes('state.pkl')
            self.wordCells.printSpikes('word.pkl')
            print 'state'
            self.printPklSpikes('state.pkl')
            print 'word'
            self.printPklSpikes('word.pkl')
        elif  (nealParameters.simulator == 'spinnaker'):
            self.stateCells.printSpikes('state.sp')
            self.wordCells.printSpikes('word.sp')

    #-- Test Functions
    def testTurnOnState(self, state, spikeGenerator):
        #self.fsa.turnOnStateSpikeSource(spikeGenerator,state,self.stateCells)
        self.fsa.turnOnStateFromSpikeSource(spikeGenerator,self.stateCells,state)
    
    def testTurnOnWord(self, word, spikeGenerator):
        self.fsa.turnOnStateFromSpikeSource(spikeGenerator,self.wordCells,word)
    

    #-----functions called externally and their subfunctions
    def createParseInputs(self):
        if self.simulator == 'nest':
            self.allocateNestInputs()
        elif self.simulator == 'spinnaker':
            self.allocateSpinnakerInputs()


    def connectInputsToWordsAndFirstState(self):
        #set up the input to turn on the first state.
        self.fsa.turnOnStateFromOneNeuron(self.InputWordSource,
                                          self.NUMBER_WORDS,self.stateCells,0)
        for wordNumber  in range (0,self.NUMBER_WORDS):
            self.fsa.turnOnStateFromOneNeuron(self.InputWordSource,wordNumber,
                                              self.wordCells,wordNumber)

    def allocateNestInputs(self):
        if self.simulator != 'nest':
            print "Error nest code should be called here"
        self.InputWordSource = nest.Create("iaf_cond_exp",
                    n=self.NUMBER_WORDS+3,params = self.fsa.CELL_PARAMS)

    def setNestInputs(self,time):
        if (not os.path.isfile("world/words.txt")):
            return
        fileHandle = open("world/words.txt",'r')
        for inpLine in fileHandle:
            splitInp = inpLine.split()
            inpTime = int(splitInp[0])
            inpNeuronNumber = int(splitInp[1])
            if (inpTime == time):
                inpNeuron = self.InputWordSource[inpNeuronNumber:
                                             inpNeuronNumber+1]
                nest.SetStatus(inpNeuron,{"V_m":-50.0})
                print inpTime, inpNeuron,"bob"
        return

        #undone apr16 test kludge.  Get input from environment
        if (time == 1500):
            inpNeuron = self.InputWordSource[self.NUMBER_WORDS:
                                             self.NUMBER_WORDS+1]
            nest.SetStatus(inpNeuron,{"V_m":-50.0})

        elif (time == 1600):
            inpNeuron = self.InputWordSource[9:10]
            nest.SetStatus(inpNeuron,{"V_m":-50.0})
        elif (time == 1700):
            inpNeuron = self.InputWordSource[6:7]
            nest.SetStatus(inpNeuron,{"V_m":-50.0})
        elif (time == 1800):
            inpNeuron = self.InputWordSource[10:11]
            nest.SetStatus(inpNeuron,{"V_m":-50.0})



    #---create inputs for spinnaker spike server
    def allocateSpinnakerInputs(self):
        if self.simulator != 'spinnaker':
            print "Error spinnaker code should be called here"

        self.InputWordSource= sim.Population(self.NUMBER_WORDS+3, #undone +1?
                    externaldevices.SpikeInjector,
                    {'port': 12345}, label="wordSpikeToBoard")
        externaldevices.activate_live_output_for(self.InputWordSource,
                    database_notify_port_num=12345)


    def recordSpinnakerInputs(self):
        #undone this fails because you can't record these.
        if self.simulator != 'spinnaker':
            print "Error spinnaker code should not be called here"
        self.InputWordSource.record()

    def printInputs(self):
        if self.simulator == "nest":
            print "undone***"
            return
            spikes = nest.GetStatus(self.stateSpikeDet)
            stateSpikes = spikes[0]['events']['times']
            print 'State Spikes', stateSpikes
            spikes = nest.GetStatus(self.wordSpikeDet)
            wordSpikes = spikes[0]['events']['times']
            print 'Word Spikes', wordSpikes
        elif self.simulator == 'spinnaker':
            self.InputWordSource.printSpikes('parseInputs.sp')


#-- Access Functions
    def getStateCells(self):
        return self.stateCells



