import sys

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from parseClass import ParseClass

RUN_DURATION = 400
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [50.0]
    inputSpikeTimes2 = [100.0]
    inputSpikeTimes3 = [150.0]
    inputSpikeTimes4 = [200.0]
    inputSpikeTimes5 = [250.0]
    inputSpikeTimes6 = [300.0]
    inputSpikeTimes7 = [350.0]
    
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}
    spikeArray6 = {'spike_times': [inputSpikeTimes6]}
    spikeArray7 = {'spike_times': [inputSpikeTimes7]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')
    spikeGen6=sim.Population(1, sim.SpikeSourceArray, spikeArray6,
                                   label='inputSpikes_6')
    spikeGen7=sim.Population(1, sim.SpikeSourceArray, spikeArray7,
                                   label='inputSpikes_7')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,
            spikeGen6, spikeGen7]

    
#spinnaker crashes if a spike source isn't connected to
#something.  Make the something here.
def makeSpinnakerSink():
    sinkCell = 0
    if (nealParameters.simulator == 'spinnaker'):
        sinkCell=sim.Population(1,sim.IF_cond_exp,{})
    return sinkCell

def turnOnSink(spikeSource,sinkCell):
    if (nealParameters.simulator == 'spinnaker'):
        connector = [(0,0,1.0, nealParameters.DELAY)]
        neal.nealProjection(spikeSource,sinkCell,connector,'inhibitory') 

def turnOnSinks(firstSink,spikeGenerators,sinkCell):
    for sinkNumber  in range (firstSink,8):
        turnOnSink(spikeGenerators[sinkNumber],sinkCell)

def selectSentence(spikeGenerators,sentenceNumber,sinkCell):
    secondSpikeGenerator =  spikeGenerators[1]
    thirdSpikeGenerator =  spikeGenerators[2]
    fourthSpikeGenerator =  spikeGenerators[3]
    fifthSpikeGenerator =  spikeGenerators[4]
    sixthSpikeGenerator =  spikeGenerators[5]
    seventhSpikeGenerator =  spikeGenerators[6]

    if (sentenceNumber == 0): # turn left .
        parser.testTurnOnWord(19,secondSpikeGenerator)
        parser.testTurnOnWord(8,thirdSpikeGenerator)
        parser.testTurnOnWord(10,fourthSpikeGenerator)
        turnOnSinks(4,spikeGenerators,sinkCell)

    elif (sentenceNumber == 1): # turn right .
        parser.testTurnOnWord(19,secondSpikeGenerator)
        parser.testTurnOnWord(13,thirdSpikeGenerator)
        parser.testTurnOnWord(10,fourthSpikeGenerator)
        turnOnSinks(4,spikeGenerators,sinkCell)

    elif (sentenceNumber == 2): # turn toward the pyramid .
        parser.testTurnOnWord(19,secondSpikeGenerator)
        parser.testTurnOnWord(18,thirdSpikeGenerator)
        parser.testTurnOnWord(16,fourthSpikeGenerator)
        parser.testTurnOnWord(11,fifthSpikeGenerator)
        parser.testTurnOnWord(10,sixthSpikeGenerator)
        turnOnSinks(6,spikeGenerators,sinkCell)

    elif (sentenceNumber == 3): # turn toward the stalactite .
        parser.testTurnOnWord(19,secondSpikeGenerator)
        parser.testTurnOnWord(18,thirdSpikeGenerator)
        parser.testTurnOnWord(16,fourthSpikeGenerator)
        parser.testTurnOnWord(14,fifthSpikeGenerator)
        parser.testTurnOnWord(10,sixthSpikeGenerator)
        turnOnSinks(6,spikeGenerators,sinkCell)

    elif (sentenceNumber == 4): # move forward .
        parser.testTurnOnWord(9,secondSpikeGenerator)
        parser.testTurnOnWord(6,thirdSpikeGenerator)
        parser.testTurnOnWord(10,fourthSpikeGenerator)
        turnOnSinks(4,spikeGenerators,sinkCell)

    elif (sentenceNumber == 5): # move back .
        parser.testTurnOnWord(9,secondSpikeGenerator)
        parser.testTurnOnWord(0,thirdSpikeGenerator)
        parser.testTurnOnWord(10,fourthSpikeGenerator)
        turnOnSinks(4,spikeGenerators,sinkCell)

    elif (sentenceNumber == 6): # move left .
        parser.testTurnOnWord(9,secondSpikeGenerator)
        parser.testTurnOnWord(8,thirdSpikeGenerator)
        parser.testTurnOnWord(10,fourthSpikeGenerator)
        turnOnSinks(4,spikeGenerators,sinkCell)

    elif (sentenceNumber == 7): # move right .
        parser.testTurnOnWord(9,secondSpikeGenerator)
        parser.testTurnOnWord(13,thirdSpikeGenerator)
        parser.testTurnOnWord(10,fourthSpikeGenerator)
        turnOnSinks(4,spikeGenerators,sinkCell)

    elif (sentenceNumber == 8): # move before the red stalactite .
        parser.testTurnOnWord(9,secondSpikeGenerator)
        parser.testTurnOnWord(2,thirdSpikeGenerator)
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(12,fifthSpikeGenerator) #red
        parser.testTurnOnWord(14,sixthSpikeGenerator) #stalactite
        parser.testTurnOnWord(10,seventhSpikeGenerator) #period
        turnOnSinks(7,spikeGenerators,sinkCell)

    elif (sentenceNumber == 9): # move before the blue stalactite .
        parser.testTurnOnWord(9,secondSpikeGenerator)
        parser.testTurnOnWord(2,thirdSpikeGenerator)
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(1,fifthSpikeGenerator) #blue
        parser.testTurnOnWord(14,sixthSpikeGenerator) #stalactite
        parser.testTurnOnWord(10,seventhSpikeGenerator) #period
        turnOnSinks(7,spikeGenerators,sinkCell)

    elif (sentenceNumber == 10): # move before the red pyramid .
        parser.testTurnOnWord(9,secondSpikeGenerator)
        parser.testTurnOnWord(2,thirdSpikeGenerator)
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(12,fifthSpikeGenerator) #red
        parser.testTurnOnWord(11,sixthSpikeGenerator) #pyramid
        parser.testTurnOnWord(10,seventhSpikeGenerator) #period
        turnOnSinks(7,spikeGenerators,sinkCell)

    elif (sentenceNumber == 11): # move before the blue pyramid .
        parser.testTurnOnWord(9,secondSpikeGenerator)
        parser.testTurnOnWord(2,thirdSpikeGenerator)
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(1,fifthSpikeGenerator) #blue
        parser.testTurnOnWord(11,sixthSpikeGenerator) #pyramid
        parser.testTurnOnWord(10,seventhSpikeGenerator) #period
        turnOnSinks(7,spikeGenerators,sinkCell)

    elif (sentenceNumber == 12): #go to the pyramid .
        parser.testTurnOnWord(7,secondSpikeGenerator) # go
        parser.testTurnOnWord(17,thirdSpikeGenerator) # to
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(11,fifthSpikeGenerator) #pyramid
        parser.testTurnOnWord(10,sixthSpikeGenerator) #period
        turnOnSinks(6,spikeGenerators,sinkCell)

    elif (sentenceNumber == 13): #go to the stalactite .
        parser.testTurnOnWord(7,secondSpikeGenerator) # go
        parser.testTurnOnWord(17,thirdSpikeGenerator) # to
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(14,fifthSpikeGenerator) #stalactite
        parser.testTurnOnWord(10,sixthSpikeGenerator) #period
        turnOnSinks(6,spikeGenerators,sinkCell)

    elif (sentenceNumber == 14): #go to the door .
        parser.testTurnOnWord(7,secondSpikeGenerator) # go
        parser.testTurnOnWord(17,thirdSpikeGenerator) # to
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(4,fifthSpikeGenerator) #door
        parser.testTurnOnWord(10,sixthSpikeGenerator) #period
        turnOnSinks(6,spikeGenerators,sinkCell)

    elif (sentenceNumber == 15): #go to the red pyramid .
        parser.testTurnOnWord(7,secondSpikeGenerator) # go
        parser.testTurnOnWord(17,thirdSpikeGenerator) # to
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(12,fifthSpikeGenerator) # red
        parser.testTurnOnWord(11,sixthSpikeGenerator) # pyramid
        parser.testTurnOnWord(10,seventhSpikeGenerator) #period
        turnOnSinks(7,spikeGenerators,sinkCell)

    elif (sentenceNumber == 16): #go to the red stalactite .
        parser.testTurnOnWord(7,secondSpikeGenerator) # go
        parser.testTurnOnWord(17,thirdSpikeGenerator) # to
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(12,fifthSpikeGenerator) # red
        parser.testTurnOnWord(14,sixthSpikeGenerator) # stalactite
        parser.testTurnOnWord(10,seventhSpikeGenerator) #period
        turnOnSinks(7,spikeGenerators,sinkCell)

    elif (sentenceNumber == 17): #go to the blue pyramid .
        parser.testTurnOnWord(7,secondSpikeGenerator) # go
        parser.testTurnOnWord(17,thirdSpikeGenerator) # to
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(1,fifthSpikeGenerator) # blue
        parser.testTurnOnWord(11,sixthSpikeGenerator) # pyramid
        parser.testTurnOnWord(10,seventhSpikeGenerator) #period
        turnOnSinks(7,spikeGenerators,sinkCell)

    elif (sentenceNumber == 18): #go to the blue stalactite .
        parser.testTurnOnWord(7,secondSpikeGenerator) # go
        parser.testTurnOnWord(17,thirdSpikeGenerator) # to
        parser.testTurnOnWord(16,fourthSpikeGenerator) #the
        parser.testTurnOnWord(1,fifthSpikeGenerator) # blue
        parser.testTurnOnWord(14,sixthSpikeGenerator) # stalactite
        parser.testTurnOnWord(10,seventhSpikeGenerator) #period
        turnOnSinks(7,spikeGenerators,sinkCell)

    elif (sentenceNumber == 19): #centre the pyramid .
        parser.testTurnOnWord(3,secondSpikeGenerator) # centre
        parser.testTurnOnWord(16,thirdSpikeGenerator) #the
        parser.testTurnOnWord(11,fourthSpikeGenerator) # pyramid
        parser.testTurnOnWord(10,fifthSpikeGenerator) #period
        turnOnSinks(5,spikeGenerators,sinkCell)

    elif (sentenceNumber == 20): #centre the stalactite .
        parser.testTurnOnWord(3,secondSpikeGenerator) # centre
        parser.testTurnOnWord(16,thirdSpikeGenerator) #the
        parser.testTurnOnWord(14,fourthSpikeGenerator) # stalactite
        parser.testTurnOnWord(10,fifthSpikeGenerator) #period
        turnOnSinks(5,spikeGenerators,sinkCell)

    elif (sentenceNumber == 21): #explore .
        parser.testTurnOnWord(5,secondSpikeGenerator) # explore
        parser.testTurnOnWord(10,thirdSpikeGenerator) #period
        turnOnSinks(3,spikeGenerators,sinkCell)

    elif (sentenceNumber == 22): #stop .
        parser.testTurnOnWord(15,secondSpikeGenerator) # stop
        parser.testTurnOnWord(10,thirdSpikeGenerator) #period
        turnOnSinks(3,spikeGenerators,sinkCell)

    else: print 'Bad Sentence Selected parseTest', sentenceNumber



#---main 
args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 2):
    sentenceToParse = int(args[1])
else:
    sentenceToParse = -1

neal = NealCoverFunctions(nealParameters.simulator)
init()
parser = ParseClass(nealParameters.simulator)

spikeGenerators = createInputs()
firstSpikeGenerator =  spikeGenerators[0]

sinkCell = makeSpinnakerSink()
parser.testTurnOnState(0,firstSpikeGenerator)
#selectSentence(spikeGenerators,0,sinkCell)
#selectSentence(spikeGenerators,1,sinkCell)
#selectSentence(spikeGenerators,22,sinkCell)
if (sentenceToParse >= 0):
    selectSentence(spikeGenerators,sentenceToParse,sinkCell)
else:
    selectSentence(spikeGenerators,1,sinkCell)


sim.run(RUN_DURATION)

parser.printResults()

